/*
                               Device
 */

#ifndef EDV_DEVICE_H
#define EDV_DEVICE_H

#include <glib.h>


typedef struct _edv_device_struct	edv_device_struct;
#define EDV_DEVICE(p)			((edv_device_struct *)(p))


/*
 *	Device Flags:
 */
typedef enum {
	EDV_DEVICE_MOUNTED		= (1 << 0),	/* Device is mounted */
	EDV_DEVICE_READ_ONLY		= (1 << 1),	/* Mount as read-only,
							 * making all objects
							 * on the device
							 * read-only */
	EDV_DEVICE_NO_UNMOUNT		= (1 << 2),	/* May not be unmounted */
	EDV_DEVICE_UNLISTED		= (1 << 3),	/* Do not show on
							 * "brief" listings */
	EDV_DEVICE_NO_SCAN		= (1 << 4),	/* Do not scan this
							 * device's contents
							 * unless absolutely
							 * nessisary (useful
							 * for NFS mounted
							 * devices) */
} edv_device_flags;

/*
 *	Device Icon States:
 */
typedef enum {
	EDV_DEVICE_ICON_STATE_STANDARD,
	EDV_DEVICE_ICON_STATE_SELECTED,		/* Selected or expanded */
	EDV_DEVICE_ICON_STATE_UNMOUNTED		/* Not mounted */
} edv_device_icon_state;
#define EDV_DEVICE_TOTAL_ICON_STATES		3

#define EDV_DEVICE_ICON_STATE_NAMES	{	\
	"Standard",				\
	"Selected",				\
	"Unmounted"				\
}  


/*
 *	Device:
 */
struct _edv_device_struct {

	edv_device_flags	flags;

	gchar		*name;		/* Descriptive name */

	gchar		*device_path;	/* Path to device object */
	gchar		*mount_path;	/* Path to mount point directory */

        gchar		*fs_type_name;  /* Conical name for the Filesystem type */
        guint32		fs_type_code;	/* Filesystem type code, one of EDV_FS_TYPE_* */

	/* Small 20x20 icons */
	gchar		*small_icon_file[EDV_DEVICE_TOTAL_ICON_STATES];

	/* Medium 32x32 icons */
	gchar		*medium_icon_file[EDV_DEVICE_TOTAL_ICON_STATES];

	/* Large 48x48 icons */
	gchar		*large_icon_file[EDV_DEVICE_TOTAL_ICON_STATES];

	/* Mount and unmount commands, if either of these are not NULL
	 * then its respective command will be issued instead of a mount
	 * performed internally with system calls by this program
	 */
	gchar		*command_mount,
			*command_unmount,
			*command_eject;

	/* Commands to run external programs for device maintainance */
	gchar		*command_check,
			*command_tools,
			*command_format;

        /* Statistics, units are in 1k blocks */
	gulong		blocks_total,
			blocks_available,	/* Available to effective user id */
			blocks_free;

	gulong		block_size;

        gulong		indicies_total,
                        indicies_free;

        guint		fs_id;

        gulong		name_length_max;

	/* Time stamps */
	gulong		last_mount_time,	/* Last time it was used */
			last_check_time;	/* Last time command_check aws runned */

};

#define EDV_DEVICE_IS_MOUNTED(p)	(((p) != NULL) ? \
 (((p)->flags & EDV_DEVICE_MOUNTED) ? TRUE : FALSE) : FALSE)

#define EDV_DEVICE_IS_READ_ONLY(p)	(((p) != NULL) ? \
 (((p)->flags & EDV_DEVICE_READ_ONLY) ? TRUE : FALSE) : FALSE)

#define EDV_DEVICE_IS_NO_UNMOUNT(p)	(((p) != NULL) ? \
 (((p)->flags & EDV_DEVICE_NO_UNMOUNT) ? TRUE : FALSE) : FALSE)

#define EDV_DEVICE_IS_UNLISTED(p)	(((p) != NULL) ? \
 (((p)->flags & EDV_DEVICE_UNLISTED) ? TRUE : FALSE) : FALSE)

#define EDV_DEVICE_IS_NO_SCAN(p)	(((p) != NULL) ? \
 (((p)->flags & EDV_DEVICE_NO_SCAN) ? TRUE : FALSE) : FALSE)


/*
 *	Matches the Device by mount path.
 *
 *	The devices_list specifies the Devices List.
 *
 *	The n specifies the return value for the matched Device's
 *	index.
 *
 *	The mount_path specifies the mount path.
 *
 *	Returns the matched Device or NULL on error.
 */
extern edv_device_struct *EDVDeviceListMatchMountPath(
	GList *devices_list,
	gint *n,
	const gchar *mount_path
);

/*
 *	Matches the Device by device path.
 *
 *	The devices_list specifies the Devices List.
 *
 *	The n specifies the return value for the matched Device's
 *	index.
 *
 *	The device_path specifies the device path.
 *
 *	Returns the matched Device or NULL on error.
 */
extern edv_device_struct *EDVDeviceListMatchDevicePath(
	GList *devices_list,
	gint *n,
	const gchar *device_path
);

/*
 *	Matches the Device by an object that exists on it.
 *
 *	The devices_list specifies the Devices List.
 *
 *	The n specifies the return value for the matched Device's
 *	index.
 *
 *	The path specifies the object.
 *
 *	Returns the matched Device or NULL on error.
 */
extern edv_device_struct *EDVDeviceListMatchObject(
	GList *devices_list,
	gint *n,
	const gchar *path
);


/*
 *	Updates the Devices' mount states with the current mount
 *	states obtained from the system.
 *
 *	Each Device's flags member's EDV_DEVICE_MOUNTED flag will
 *	be either set or unset.
 *
 *	The devices_list specifies the Devices List.
 */
extern void EDVDevicesListUpdateMountStates(GList *devices_list);

/*
 *	Updates the Device's mount state with the current mount
 *	state obtained from the system.
 *
 *	The Device's flags member's EDV_DEVICE_MOUNTED flag will
 *	be either set or unset.
 *
 *	The d specifies the Device.
 */
extern void EDVDeviceUpdateMountState(edv_device_struct *d);


/*
 *	Updates the Devices' statistics with the statistics obtained
 *	from the system.
 *
 *	Each Device's blocks_total, blocks_available, blocks_free,
 *	block_size, indicies_total, indicies_free, fs_id, and
 *	name_length_max members will be updated by this call.
 *
 *	EDVDevicesListUpdateMountStates() should always be called
 *	prior to this call to ensure that the mount states are up
 *	to date.
 *
 *	The devices_list specifies the Devices List.
 */
extern void EDVDevicesListUpdateStats(GList *devices_list);

/*
 *	Updates the Device's statistics with the statistics obtained
 *	from the system.
 *
 *	The Device's blocks_total, blocks_available, blocks_free,
 *	block_size, indicies_total, indicies_free, fs_id, and
 *	name_length_max members will be updated by this call.
 *
 *	EDVDeviceUpdateMountState() should always be called prior to
 *	this call to ensure that the mount states are up to date.
 *
 *	The d specifies the Device.
 */
extern void EDVDeviceUpdateStats(edv_device_struct *d);


/*
 *	Creates a new Device.
 */
extern edv_device_struct *EDVDeviceNew(void);

/*
 *	Coppies the Device.
 */
extern edv_device_struct *EDVDeviceCopy(edv_device_struct *d);

/*
 *	Deletes the Device.
 */
extern void EDVDeviceDelete(edv_device_struct *d);


#endif	/* EDV_DEVICE_H */
